// node_modules

const http = require('http');
const https = require('https');
const url = require('url');
const fs = require('fs');
const path = require('path');
const net = require('net');

var WebSocketServer = require('ws').Server;

// =========================================

var RESPONSE_PORT;

// =========================================
// Server
// =========================================
var port = 8082; // localhost:8082

var server = new http.Server();
server.listen(port);
server.on('request', function(req, res){
    // req - содержит инф-ю, кот-ю присылает браузер, в т.ч. url пришедшего запроса
    // res - объект ответа
    // из первого читаем, во второй пишем
    if(req.method != "GET")
        return;
    var urlParsed = url.parse(req.url, true);

    switch( urlParsed.pathname ){
        case "/favicon.ico":
            return sendFile('/microphone-16.png', res);
        case "":
        case "/":
        case "/index.html":
            return sendFile('/index.html', res);
        case "/do_capture":
            console.log("server->do_capture");

            res.end();

            if( urlParsed.query.response_port && !isNaN(urlParsed.query.response_port) )
                RESPONSE_PORT = parseInt( urlParsed.query.response_port );
            console.log('RESPONSE_PORT: '+RESPONSE_PORT);

            if( !CLIENTS.length )
                return sendMessageByPort("[captured_error]: not active webSocket-clients");

            return ws_send({do_capture: true, test_text: decodeURI(urlParsed.query.test_text||'')});
    }

    sendFile( urlParsed.pathname, res );
});

// =========================================
// webSocketServer
// =========================================
var CLIENTS = [];
function ws_send(json){
    for(var i = 0; i < CLIENTS.length; i++){
        /*CLIENTS[i].send(JSON.stringify(json), function (error) {
            console.error(error);
        })*/
        CLIENTS[i].send(JSON.stringify(json), {}, function (error, data) {
            console.error(error);
        })
    }
}
// IE ниже v10 не поддерживает WebSocket
var webSocketServer = new WebSocketServer({server: server});
webSocketServer.on('connection', function(ws) {
    CLIENTS.push(ws);

    ws.on('message', function(message) {
        console.log(message);
        message = JSON.parse(message);

        if(message.captured_transcript){
            let transcript = message.captured_transcript.toLowerCase();
            //console.log("captured_transcript: " + transcript);

            sendMessageByPort( "[captured_transcript]:" + transcript );
        }
        else if( message.captured_error ){
            //console.log("captured_error: " + message.captured_error);
            sendMessageByPort( "[captured_error]:" + message.captured_error );
        }
    });

    ws.on('close', function () {
        for(var i = 0; i < CLIENTS.length; i++){
            if(CLIENTS[i] == ws){
                CLIENTS.splice(i,1);
                break;
            }
        }
        console.log("WebSocketServer: клиент отключился");
    });

    console.log("WebSocketServer: клиент подключился");
});

// =========================================
// client
// =========================================
function sendMessageByPort(message) {
    if( !RESPONSE_PORT )
        return console.error('RESPONSE_PORT not defined');

    var client = net.connect({port: RESPONSE_PORT}, function() {
        console.log('Client connected by port ' + RESPONSE_PORT);

        client.on('data', function(data) {
            console.log(data.toString());
            client.end();
        });
        client.on('end', function() {
            console.log('Client disconnected.');
        });

        client.write( encodeURI(message) );
        client.end();
    });

    client.on('error', function(error) {
        console.error("sendMessageByPort(): " + error);
    });
}

// =========================================
// =========================================

function sendFile(filePath, res, callback){
    fs.readFile(__dirname + filePath, //'/index.html',
        function (err, data) {
            if (err) {
                res.writeHead(500);
                return res.end('Error loading ' + filePath);
            }

            res.writeHead(200);
            res.end(data);

            if( callback )
                callback(err, data);
        });
}

console.log("Сервер запущен");